# -*- encoding: utf-8 -*-
# -----------------------------------------------------------------------------
# @File Name  : order_service.py
# @Time       : 2020/11/18 下午3:35
# @Author     : X. Peng
# @Email      : acepengxiong@163.com
# @Software   : PyCharm
# -----------------------------------------------------------------------------
import time

from app.api.engine import tamp_pay_session, tamp_user_session
from app.model.account_balance import AccountBalance
from app.model.account_topup_order import AccountTopupOrder
from app.model.curriculum_order import CurriculumOrder
from app.model.tamp_user_models import CurriculumPrice, CurriculumColumn, CurriculumRes


class TopUpOrderService:
    """"""
    def __init__(self):
        pass

    def get_order(self, args):
        res = tamp_pay_session.query(AccountTopupOrder).filter(AccountTopupOrder.user_id==args['user_id']).all()
        return [r.to_dict() for r in res]


    def create_order(self, args):
        args['order_no'] = time.strftime('%Y%m%d', time.localtime(time.time())) + str(int(time.time() * 100000))
        args['id'] = AccountTopupOrder.__tablename__ + str(int(time.time() * 100000))
        args['amount'] = args['amount'] * 100
        order_info = AccountTopupOrder(**args)
        tamp_pay_session.add(order_info)
        tamp_pay_session.commit()
        tamp_pay_session.close()
        return True


class CurriculumOrderService:
    """"""
    def __init__(self):
        pass

    def get_order(self, args):
        """."""
        curriculum_column = tamp_user_session.query(CurriculumColumn).all()
        curriculum_res = tamp_user_session.query(CurriculumRes).all()
        res = tamp_pay_session.query(CurriculumOrder).filter(CurriculumOrder.user_id == args['user_id']).all()
        curriculum_column = {r.id: r.to_dict() for r in curriculum_column}
        curriculum_res = {r.id: r.to_dict() for r in curriculum_res}
        orders = [r.to_dict() for r in res]
        for order in orders:
            if order['prod_type'] == '1':
                order['prod_info'] = curriculum_column.get(order['prod_id'], None)
            elif order['prod_type'] in ['3', '4', '5']:
                order['prod_info'] = curriculum_res.get(order['prod_id'], None)
        return orders


    def create_order(self, args):
        args['order_no'] = time.strftime('%Y%m%d', time.localtime(time.time())) + str(int(time.time() * 100000))
        amount = tamp_user_session.query(CurriculumPrice.price).filter(CurriculumPrice.rel_id == args['prod_id']).first()[0]
        args['id'] = CurriculumOrder.__tablename__ + str(int(time.time()*100000))
        args['amount'] = amount * args['prod_quantity']
        order_info = CurriculumOrder(**args)
        tamp_pay_session.add(order_info)
        tamp_pay_session.commit()
        return True


def topUpSuccessAction(order_no):
    """."""
    topup_order = tamp_pay_session.query(AccountTopupOrder).filter(AccountTopupOrder.order_no == order_no).first()
    # 幂等校验
    if topup_order.order_status == 1:
        return False
    tamp_pay_session.query(AccountTopupOrder).filter(AccountTopupOrder.order_no == order_no).update({
        'order_status': 1
    })
    # 账户充值
    account_topup_order = tamp_pay_session.query(AccountTopupOrder).filter(AccountTopupOrder.order_no == order_no).first()
    if account_topup_order.pay_method == 4:
        # 苹果支付
        if not tamp_pay_session.query(AccountBalance).filter(AccountBalance.user_id == account_topup_order.user_id).all():
            tamp_pay_session.add(AccountBalance(apple_balance=account_topup_order.amount))
        else:
            tamp_pay_session.query(AccountBalance).filter(AccountBalance.user_id == account_topup_order.user_id).update({
                'apple_balance': AccountBalance.apple_balance + account_topup_order.amount
            })
    else:
        # 支付宝，微信支付
        if not tamp_pay_session.query(AccountBalance).filter(AccountBalance.user_id == account_topup_order.user_id).all():
            tamp_pay_session.add(AccountBalance(android_balance=account_topup_order.amount))
        else:
            tamp_pay_session.query(AccountBalance).filter(AccountBalance.user_id == account_topup_order.user_id).update({
                'apple_balance': AccountBalance.android_balance + account_topup_order.amount
            })
    tamp_pay_session.commit()



def consumeSucessAction(order_no):
    """."""
    curriculum_order = tamp_pay_session.query(CurriculumOrder.order_no == order_no).all()
    # 幂等校验
    if curriculum_order.order_status == 1:
        return False
    # tamp_pay_session.query(CurriculumOrder).filter(CurriculumOrder.order_no == )